div.subscribe-to-parent {
    background-color: #FFFFE3;
    border: 1px solid #999966;
    margin-bottom: 1em;
    padding: 0.25em;
}

div.subscribe-to-parent div {
    margin-bottom: 0.25em;
}

div.subscribe-notifiers {
    margin-bottom: 1em;
}
